// $Id$

/* Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Girish Ramakrishnan.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Girish Ramakrishnan <gramakri@uiuc.edu>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 */

var gInThunderbird = false;

var gucPrefs = Components.classes["@mozilla.org/preferences-service;1"].
             getService(Components.interfaces.nsIPrefService).getBranch("guc.");

var gucStringBundle;

var observerService = Components.classes["@mozilla.org/observer-service;1"].
             getService(Components.interfaces.nsIObserverService);

// register callback everytime we are created
window.addEventListener("load", gucInit, false);

var observer = {
    observe: function(subject, topic, data) {
        gucCreateContextMenu();
    }
};

function gucInit()
{
    if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
        gInThunderbird = true;
    } 

    gucStringBundle = document.getElementById("guc_string_bundle");

    if (document.getElementById("contentAreaContextMenu")) {
        document.getElementById("contentAreaContextMenu").
                           addEventListener("popupshowing", onGucPopup, false);
    } else if (document.getElementById("messagePaneContext")) {
        document.getElementById("messagePaneContext").
                           addEventListener("popupshowing", onGucPopup, false);
    } else if (document.getElementById("msgComposeContext")) {
        document.getElementById("msgComposeContext").
                           addEventListener("popupshowing", onGucPopup, false);
    }

    if (!gucPrefs.prefHasUserValue("contextMenuText")) {
        gucPrefs.setCharPref("contextMenuText", 
                   gucStringBundle.getString("guc.default.contextMenuText"));
    }

    if (!gucPrefs.prefHasUserValue("menutext1")) {
        var i = 1;
        var mt;
        while (true) {
           try { mt = gucStringBundle.getString("guc.default.menutext" + i);  }
           catch (ex) { break; }
           if (mt == "") break;
           gucPrefs.setCharPref("menutext"+i, mt);
           var url = gucStringBundle.getString("guc.default.url" + i);
           gucPrefs.setCharPref("url"+i, url);
           i = i + 1;
        }
    }
   
    observerService.addObserver(observer, "guc-settings", false);

    gucCreateContextMenu();
}

function gucCreateContextMenu()
{
    // destroy and create the menupopup that holds the guc urls
    var m = document.getElementById("context-menu-guc");
    if (m == null) return;
    var p = document.getElementById("context-guc-urls");
    if (p != null) m.removeChild(p);
    p = document.createElement("menupopup");
    p.setAttribute("id", "context-guc-urls");
    m.appendChild(p);

    var i = 1;

    if (gucPrefs.getCharPref("contextMenuText") != "") {
        while (true) {
            if (!gucPrefs.prefHasUserValue("menutext"+i)) break;
            text = gucPrefs.getCharPref("menutext" + i);
            if (text == "") break;
            m = document.createElement("menuitem");
            m.setAttribute("id", "guc_menuitem" + i);
            m.setAttribute("oncommand", "onGucClick("+i+")");
            m.setAttribute("label", "null");
            p.appendChild(m);
            i = i + 1;
        }
    }

    // someday, this will work correctly :(
    m.hidden = (i==1);
    document.getElementById("context-sep-guc").hidden = (i==1);
}

// a small helper function to help us replace t in s with u
function replace(s, t, u) 
{
    var i, r;
    i = s.indexOf(t);
    r = "";
    if (i == -1) return s;
    r += s.substring(0,i) + u;
    if (i + t.length < s.length)
        r += replace(s.substring(i + t.length, s.length), t, u);
    return r;
}

// invoked when the menu is just about to popup
function onGucPopup()
{
    var i = 1;
    var selectedText = gucGetSelectedText(" ");
    var m = document.getElementById("context-menu-guc");

    if (selectedText != "") {
        var item, menuText;
        var words = selectedText.split(" ");

        m.hidden = false;
        document.getElementById("context-sep-guc").hidden = false;
        menuText = gucPrefs.getCharPref("contextMenuText");
        for(j=1; j<=words.length; j++)
               menuText = replace(menuText, "$"+j, words[j-1]);
        menuText = replace(menuText, "$$", selectedText);

        if (menuText.length > 30)
            menuText = menuText.substr(menuText, 42) + "...";

        m.setAttribute("label", menuText);
        
        while (true) {
            if (!gucPrefs.prefHasUserValue("menutext"+i)) break;
            menuText = gucPrefs.getCharPref("menutext" + i);
            item = document.getElementById("guc_menuitem" + i);
            if (item == null) break;
            for(j=1; j<=words.length; j++)
               menuText = replace(menuText, "$"+j, words[j-1]);

            menuText = replace(menuText, "$$", selectedText);
            item.setAttribute("label", menuText);
            i = i + 1;
        }
    } else { // simply right clicked with no text
        m.hidden = true;
        document.getElementById("context-sep-guc").hidden = true;
    }
}

// Will get the selected text, format it and pass it back with joinChar seperating 
// individual words
function gucGetSelectedText(joinChar)
{
    var node = document.popupNode;
    var selection = "";
    var nodeLocalName = node.localName.toUpperCase();
    if ((nodeLocalName == "TEXTAREA") || (nodeLocalName == "INPUT" && node.type == "text")) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } 
    else {
        var focusedWindow = document.commandDispatcher.focusedWindow;
        selection = focusedWindow.__proto__.getSelection.call(focusedWindow).toString();
    }

    // replace all the \r \n \t with spaces
    selection = selection.replace(/(\n|\r|\t)+/g, " ");
    // remove leading and trailing spaces
    selection = selection.replace(/(^\s+)|(\s+$)/g, "");

    selection = selection.split(" ");

    // remove &|[]" and others from beginning and end of every word
    for (i=0; i<selection.length; i++) {
        selection[i]=selection[i].replace(/^(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+/, "");
        selection[i]=selection[i].replace(/(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+$/, "");
    }

    selection = selection.join(joinChar);
    return selection; 
}

// Called when one of the url's is clicked
function onGucClick(n)
{
    var url = gucPrefs.getCharPref("url" + n);
    var selectedText = gucGetSelectedText(" ");
    var words = selectedText.split(" ");
    
    for(j=1; j<=words.length; j++)
         url = replace(url, "$"+j, words[j-1]);

    url = replace(url, "$$", replace(selectedText, " ", "%20"));
    url = replace(url, "$+", replace(selectedText, " ", "+"));

    if (gInThunderbird) {
      var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();         
      messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
      messenger.launchExternalURL(url);
    }
    else openNewTabWith(url, null, null, true);
}

